@@ -15,7 +15,7 @@
 #include "generator.h"
 #include "kscreen_daemon_debug.h"
 #include "kscreenadaptor.h"
-#include "osdmanager.h"
+#include "osdservice_interface.h"
 
 #include <kscreen/configmonitor.h>
 #include <kscreen/getconfigoperation.h>
@@ -29,12 +29,45 @@
 #include <KPluginFactory>
 
 #include <QAction>
+#include <QGuiApplication>
 #include <QOrientationReading>
 #include <QShortcut>
 #include <QTimer>
 
+#if HAVE_X11
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+#include <QX11Info>
+#else
+#include <private/qtx11extras_p.h>
+#endif
+#include <X11/Xatom.h>
+#include <X11/Xlib-xcb.h>
+#include <X11/extensions/XInput.h>
+#include <X11/extensions/XInput2.h>
+#endif
+
 K_PLUGIN_CLASS_WITH_JSON(KScreenDaemon, "kscreen.json")
 
+#if HAVE_X11
+struct DeviceListDeleter {
+    void operator()(XDeviceInfo *p)
+    {
+        if (p) {
+            XFreeDeviceList(p);
+        }
+    }
+};
+
+struct XDeleter {
+    void operator()(void *p)
+    {
+        if (p) {
+            XFree(p);
+        }
+    }
+};
+#endif
+
 KScreenDaemon::KScreenDaemon(QObject *parent, const QList<QVariant> &)
     : KDEDModule(parent)
     , m_monitoring(false)
@@ -47,6 +80,7 @@ KScreenDaemon::KScreenDaemon(QObject *parent, const QList<QVariant> &)
     connect(m_orientationSensor, &OrientationSensor::valueChanged, this, &KScreenDaemon::updateOrientation);
 
     KScreen::Log::instance();
+    qMetaTypeId<KScreen::OsdAction>();
     QMetaObject::invokeMethod(this, "getInitialConfig", Qt::QueuedConnection);
 }
 
@@ -83,8 +117,12 @@ void KScreenDaemon::init()
     connect(action, &QAction::triggered, this, &KScreenDaemon::displayButton);
 
     new KScreenAdaptor(this);
-    // Initialize OSD manager to register its dbus interface
-    m_osdManager = new KScreen::OsdManager(this);
+
+    const QString osdService = QStringLiteral("org.kde.kscreen.osdService");
+    const QString osdPath = QStringLiteral("/org/kde/kscreen/osdService");
+    m_osdServiceInterface = new OrgKdeKscreenOsdServiceInterface(osdService, osdPath, QDBusConnection::sessionBus(), this);
+    // Set a longer timeout to not assume timeout while the osd is still shown
+    m_osdServiceInterface->setTimeout(std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::seconds(60)).count());
 
     m_changeCompressor->setInterval(10);
     m_changeCompressor->setSingleShot(true);
@@ -248,6 +286,20 @@ void KScreenDaemon::setAutoRotate(bool value)
     m_orientationSensor->setEnabled(value);
 }
 
+void KScreenDaemon::showOSD()
+{
+    auto call = m_osdServiceInterface->showActionSelector();
+    auto watcher = new QDBusPendingCallWatcher(call);
+    connect(watcher, &QDBusPendingCallWatcher::finished, this, [this, watcher] {
+        watcher->deleteLater();
+        QDBusReply<int> reply = *watcher;
+        if (!reply.isValid()) {
+            return;
+        }
+        applyOsdAction(static_cast<KScreen::OsdAction::Action>(reply.value()));
+    });
+}
+
 void KScreenDaemon::applyOsdAction(KScreen::OsdAction::Action action)
 {
     switch (action) {
@@ -286,10 +338,9 @@ void KScreenDaemon::applyIdealConfig()
 
     if (showOsd) {
         qCDebug(KSCREEN_KDED) << "Getting ideal config from user via OSD...";
-        auto action = m_osdManager->showActionSelector();
-        connect(action, &KScreen::OsdAction::selected, this, &KScreenDaemon::applyOsdAction);
+        showOSD();
     } else {
-        m_osdManager->hideOsd();
+        m_osdServiceInterface->hideOsd();
     }
 }
 
@@ -321,8 +372,170 @@ void KScreenDaemon::configChanged()
         connect(m_saveTimer, &QTimer::timeout, this, &KScreenDaemon::saveCurrentConfig);
     }
     m_saveTimer->start();
+#if HAVE_X11
+    alignX11TouchScreen();
+#endif
 }
 
+#if HAVE_X11
+void KScreenDaemon::alignX11TouchScreen()
+{
+    if (qGuiApp->platformName() != QStringLiteral("xcb")) {
+        return;
+    }
+    auto *display = QX11Info::display();
+    if (!display) {
+        return;
+    }
+    auto *connection = QX11Info::connection();
+    if (!connection) {
+        return;
+    }
+
+    const QRect totalRect(QPoint(0, 0), m_monitoredConfig->data()->screen()->currentSize());
+    QRect internalOutputRect;
+    int touchScreenRotationAngle = 0;
+
+    for (const auto &output : m_monitoredConfig->data()->connectedOutputs()) {
+        if (output->isEnabled() && output->type() == KScreen::Output::Panel) {
+            internalOutputRect = output->geometry();
+
+            switch (output->rotation()) {
+            case KScreen::Output::Left:
+                touchScreenRotationAngle = 90;
+                break;
+            case KScreen::Output::Right:
+                touchScreenRotationAngle = 270;
+                break;
+            case KScreen::Output::Inverted:
+                touchScreenRotationAngle = 180;
+                break;
+            default:
+                touchScreenRotationAngle = 0;
+            }
+        }
+    }
+
+    // Compute the transformation matrix for the
+    QTransform transform;
+    transform = transform.translate(float(internalOutputRect.x()) / float(totalRect.width()), float(internalOutputRect.y()) / float(totalRect.height()));
+    transform = transform.scale(float(internalOutputRect.width()) / float(totalRect.width()), float(internalOutputRect.height()) / float(totalRect.height()));
+    transform = transform.rotate(touchScreenRotationAngle);
+
+    // After rotation we need to make the matrix origin aligned with the workspace again
+    // ____                                                      ___
+    // |__|  -> 90° clockwise -> ___  -> needs to be moved up -> | |
+    //                           | |                             |_|
+    //                           |_|
+    switch (touchScreenRotationAngle) {
+    case 90:
+        transform = transform.translate(0, -1);
+        break;
+    case 270:
+        transform = transform.translate(-1, 0);
+        break;
+    case 180:
+        transform = transform.translate(-1, -1);
+        break;
+    default:
+        break;
+    }
+
+    auto getAtom = [](xcb_connection_t *connection, const char *name) {
+        auto cookie = xcb_intern_atom(connection, true, strlen(name), name);
+        auto reply = xcb_intern_atom_reply(connection, cookie, nullptr);
+        if (reply) {
+            return reply->atom;
+        } else {
+            return xcb_atom_t(0);
+        }
+    };
+
+    int nDevices = 0;
+    std::unique_ptr<XDeviceInfo, DeviceListDeleter> deviceInfo(XListInputDevices(display, &nDevices));
+    auto touchScreenAtom = getAtom(connection, XI_TOUCHSCREEN);
+    if (touchScreenAtom == 0) {
+        return;
+    }
+    auto matrixAtom = getAtom(connection, "Coordinate Transformation Matrix");
+    if (matrixAtom == 0) {
+        return;
+    }
+    auto calibrationMatrixAtom = getAtom(connection, "libinput Calibration Matrix");
+    auto floatAtom = getAtom(connection, "FLOAT");
+    if (floatAtom == 0) {
+        return;
+    }
+
+    auto setMatrixAtom = [display, floatAtom](XDeviceInfo *info, Atom atom, const QTransform &transform) {
+        Atom type;
+        int format = 0;
+        unsigned long nItems, bytesAfter;
+        unsigned char *dataPtr = nullptr;
+
+        std::unique_ptr<unsigned char, XDeleter> data(dataPtr);
+        XIGetProperty(display, info->id, atom, 0, 1000, False, AnyPropertyType, &type, &format, &nItems, &bytesAfter, &dataPtr);
+
+        if (nItems != 9) {
+            return;
+        }
+        if (format != sizeof(float) * CHAR_BIT || type != floatAtom) {
+            return;
+        }
+
+        float *fData = reinterpret_cast<float *>(dataPtr);
+
+        fData[0] = transform.m11();
+        fData[1] = transform.m21();
+        fData[2] = transform.m31();
+
+        fData[3] = transform.m12();
+        fData[4] = transform.m22();
+        fData[5] = transform.m32();
+
+        fData[6] = transform.m13();
+        fData[7] = transform.m23();
+        fData[8] = transform.m33();
+
+        XIChangeProperty(display, info->id, atom, type, format, PropModeReplace, dataPtr, nItems);
+    };
+
+    for (XDeviceInfo *info = deviceInfo.get(); info < deviceInfo.get() + nDevices; info++) {
+        // Make sure device is touchscreen
+        if (info->type != touchScreenAtom) {
+            continue;
+        }
+
+        int nProperties = 0;
+        std::unique_ptr<Atom, XDeleter> properties(XIListProperties(display, info->id, &nProperties));
+
+        bool matrixAtomFound = false;
+        bool libInputCalibrationAtomFound = false;
+
+        Atom *atom = properties.get();
+        Atom *atomEnd = properties.get() + nProperties;
+        for (; atom != atomEnd; atom++) {
+            if (!internalOutputRect.isEmpty() && *atom == matrixAtom) {
+                matrixAtomFound = true;
+            } else if (!internalOutputRect.isEmpty() && *atom == calibrationMatrixAtom) {
+                libInputCalibrationAtomFound = true;
+            }
+        }
+
+        if (libInputCalibrationAtomFound) {
+            setMatrixAtom(info, calibrationMatrixAtom, transform);
+        }
+        if (matrixAtomFound) {
+            setMatrixAtom(info, matrixAtom, libInputCalibrationAtomFound ? QTransform() : transform);
+        }
+
+        // For now we assume there is only one touchscreen
+        XFlush(display);
+        break;
+    }
+}
+#endif
+
 void KScreenDaemon::saveCurrentConfig()
 {
     qCDebug(KSCREEN_KDED) << "Saving current config to file";
@@ -342,9 +555,7 @@ void KScreenDaemon::saveCurrentConfig()
 void KScreenDaemon::displayButton()
 {
     qCDebug(KSCREEN_KDED) << "displayBtn triggered";
-
-    auto action = m_osdManager->showActionSelector();
-    connect(action, &KScreen::OsdAction::selected, this, &KScreenDaemon::applyOsdAction);
+    showOSD();
 }
 
 void KScreenDaemon::lidClosedChanged(bool lidIsClosed)
