@@ -91,6 +91,7 @@ void Output::readInGlobalPartFromInfo(KScreen::OutputPtr output, const QVariantM
     output->setVrrPolicy(config.vrrPolicy.value_or(KScreen::Output::VrrPolicy::Automatic));
     output->setOverscan(config.overscan.value_or(0));
     output->setRgbRange(config.rgbRange.value_or(KScreen::Output::RgbRange::Automatic));
+    output->setEnabled(true); // gets disabled later if necessary
 
     KScreen::ModePtr matchingMode;
     if (config.modeId) {
@@ -98,8 +99,8 @@ void Output::readInGlobalPartFromInfo(KScreen::OutputPtr output, const QVariantM
     }
     if (!matchingMode) {
         qCWarning(KSCREEN_KDED) << "\tFailed to find a matching mode - this means that our config is corrupted"
-                                   "or a different device with the same serial number has been connected (very unlikely)."
-                                   "Falling back to preferred modes.";
+                                   " or a different device with the same serial number has been connected (very unlikely)."
+                                   " Falling back to preferred modes.";
         matchingMode = output->preferredMode();
     }
     if (!matchingMode) {
@@ -351,14 +352,18 @@ void Output::readInOutputs(KScreen::ConfigPtr config, const QVariantList &output
     // to be able to tell apart multiple identical outputs, these need special treatment
     QStringList duplicateIds;
     {
-        QStringList allIds;
-        allIds.reserve(outputs.count());
+        QStringList allConnectedIds;
+        allConnectedIds.reserve(outputs.count());
         for (const KScreen::OutputPtr &output : outputs) {
+            if (!output->isConnected()) {
+                // Duplicated IDs only matter if the duplicates are actually connected. Duplicates may also be transient.
+                continue;
+            }
             const auto outputId = output->hash();
-            if (allIds.contains(outputId) && !duplicateIds.contains(outputId)) {
+            if (allConnectedIds.contains(outputId) && !duplicateIds.contains(outputId)) {
                 duplicateIds << outputId;
             }
-            allIds << outputId;
+            allConnectedIds << outputId;
         }
     }
 
@@ -392,7 +397,7 @@ void Output::readInOutputs(KScreen::ConfigPtr config, const QVariantList &output
             // no info in info for this output, try reading in global output info at least or set some default values
 
             qCWarning(KSCREEN_KDED) << "\tFailed to find a matching output in the current info data - this means that our info is corrupted"
-                                       "or a different device with the same serial number has been connected (very unlikely).";
+                                       " or a different device with the same serial number has been connected (very unlikely).";
             if (!readInGlobal(output)) {
                 // set some default values instead
                 readInGlobalPartFromInfo(output, QVariantMap());
